--[[
	battleBar -  A bar for holding Pet Battle Buttons
--]]

local AddonName, Addon = ...

-- register buttons for use later
local battleButtons = {}


--[[ Battle Pet Bar ]]--

local BattleBar = Addon:CreateClass('Frame', Addon.ButtonBar)

function BattleBar:New()
	return BattleBar.proto.New(self, 'bags')
end

function BattleBar:GetDefaults()
	return {
		point = 'BOTTOMRIGHT',
		spacing = 2,
	}
end



--[[ Frame Overrides ]]--

function BattleBar:GetButton(index)
	return battleButtons[index]
end

function BattleBar:NumButtons()
	return #battleButtons
end

function BattleBar:CreateMenu()
	local menu = Addon:NewMenu(self.id)
	local L = LibStub('AceLocale-3.0'):GetLocale('Dominos-Config')

	local layoutPanel = menu:NewPanel(L.Layout)


	layoutPanel:AddLayoutOptions()

	menu:AddAdvancedPanel()

	self.menu = menu
end

--[[ Battle Pet Bar Controller ]]

local BattleBarController = Addon:NewModule('BattleBar')

function BattleBarController:OnInitialize()
	for slot = (NUM_BAG_SLOTS - 1), 0, -1 do
		self:RegisterButton(('CharacterBag%dSlot'):format(slot))
	end

	self:RegisterButton('MainMenuBarBackpackButton')
end

function BattleBarController:OnEnable()
	for i, button in pairs(battleButtons) do
		Addon:GetModule('ButtonThemer'):Register(button, 'Battle Pet Bar', { Icon = button.icon, Border = button.IconBorder })
	end
end

function BattleBarController:Load()
	self.frame = BattleBar:New()
end

function BattleBarController:Unload()
	if self.frame then
		self.frame:Free()
		self.frame = nil
	end
end

function BattleBarController:RegisterButton(name)
	local button = _G[name]

	table.insert(battleButtons, button)
end
