local bar = _G[...]
local Menu = {}
bar.Menu = Menu

local NAME = 1

function Menu.Slider(panel, name, key, min, max, step)
	local slider =  panel:NewSlider({
		name = name,
		min = min,
		max = max,
		get = function(self) --Getter
			return panel.owner.sets[panel.name][key]
		end,
		set = function(self) --Setter
			local owner = panel.owner
			owner.sets[panel.name][key] = self:GetValue()
			owner:Layout()
			return owner.sets[panel.name][key]
		end,
	})
	slider.height = 30
	slider:SetWidth(280)
	return slider, "slider"
end

function Menu.CheckButton(panel, name, key)
	local c = panel:NewCheckButton{
		name = name,
		get = function() return panel.owner.sets[panel.name][key] end,
		set = function(self, enable)
			local owner = panel.owner
			owner.sets[panel.name][key] = self:GetChecked() or false
			owner:Layout()
			
		end
	}
	c.height = 28
	return c, "checkbutton"
end

function Menu.Button(panel, name, isColor, handler, func)
	local b = CreateFrame("Button", NAME..name, panel, "UIMenuButtonStretchTemplate")
	NAME = NAME + 1
	b:SetSize(140, 25)
	b:SetText(name)
	b.text = b:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	b.text:SetDrawLayer(BACKGROUND)
	if isColor then
		b.texture = b:CreateTexture(nil, "BACKGROUND")
		b.texture:SetDrawLayer("BACKGROUND", -8)
		b.texture:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
	else
		b.texture = CreateFrame("Frame", b:GetName().."texture", b)
		b.texture:SetFrameLevel(1)
	end
	b.text:SetDrawLayer("ARTWORK", 7)
	if handler then
		b:SetScript("OnClick", function()
			func(panel.owner[handler])
		end)
	end
	b.texture:SetPoint("TopLeft", b, "BottomLeft")
	b.texture:SetPoint("TopRight", b, "BottomRight")
	b.texture:SetHeight(25)
	b.text:SetAllPoints(b.texture)
	panel.height = panel.height + 27
	b.height = 27
	local prev = panel.lastWidget
	if prev then
		b:SetPoint("TopLeft", panel.lastWidget, "BottomLeft", 0, -2)
	else
		b:SetPoint("TOPLEFT", 2, -2)
	end
	panel.lastWidget = b
	return b, "checkbutton"
end

function Menu.EditBox(p, name, key, setter)
	local editBox = CreateFrame("EditBox", NAME .. name, p,  "InputBoxTemplate")
	NAME = NAME + 1
	editBox:SetHeight(20)
	editBox:SetAutoFocus(false)
	local set = CreateFrame("Button", editBox:GetName().."Set", p, "UIPanelButtonTemplate")
	local flip, flop = name, "Set"
	local hold = false
	local isLocked = false
	local function flipText(why)
		local text = flip
		if why == "lock" then 
			isLocked = true
		elseif why == "unlock" then 
			isLocked = false
		end
		if isLocked == true then
			text = flop
		else
			text = flip
		end
		if (why == "OnEnter") then
			if isLocked == true then
				text = flip
			else
				text = flop
			end
		elseif (why == "OnLeave")  then
			if isLocked == true then
				text = flop
			else
				text = flip
			end
		end 
		set:SetText(text)
	end
	set:SetScript("OnEnter", function(self)
		flipText("OnEnter")
	end)
	set:SetScript("OnLeave", function(self)
		flipText("OnLeave")
	end)
	local function setText(self)
		if type(key) == "function" then
			self:SetText(key(self:GetParent().owner) or "")
		else
			self:SetText(self:GetParent().owner.sets[p.name][key] or "")
		end
	end
	
	editBox:SetScript("OnShow", function(self)
	
		setText(self)
	end)
	editBox:SetScript("OnEscapePressed", function(self)
		setText(self)
		self:ClearFocus()
	end)
	editBox:SetScript("OnEnterPressed", function(self)
		local text = self:GetText()
		setter(self:GetParent().owner, text ~= "" and text or nil)
		setText(self)
		self:ClearFocus()
	end)
	editBox:SetScript("OnEditFocusLost", function(self) self:HighlightText(0, 0) flipText("unlock") end)
	editBox:SetScript("OnEditFocusGained", function(self) self:HighlightText() flipText("lock") end)
	set:SetHeight(20)
	set:SetText(name)
	set:SetScript("OnClick", function(self)
		editBox:GetScript("OnEnterPressed")(editBox)
	end)
	editBox:SetPoint("Left", set, "Right", 0, 0)
	editBox:SetPoint("Right", p)
	local prev = p.checkbutton
	if prev then
		set:SetPoint("TOP", prev, "BOTTOM", 0, -2)
	else
		set:SetPoint("TOPLEFT", 3, 0)
	end
	p.checkbutton = set
	set.height = 20
	p.height = p.height + 20
	return p, "slider"
end

function Menu.Menu(panel, name, key, table)
	local f = CreateFrame("Frame", NAME..name, panel)
	NAME = NAME + 1
	f:SetSize(24, 24)
	f.button = CreateFrame("Button", f:GetName().."Button", f)
	f.button:SetPoint("Top")
	f.button:SetSize(24, 24)
	f.button:SetNormalTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up")
	f.button:SetPushedTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Down")
	f.button:SetDisabledTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Disabled")
	f.button:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight")
	f.text = f:CreateFontString(f:GetName() .. "Text", "OVERLAY", "GameFontHighlightSmall")
	f.text:SetPoint("BottomLeft", f.button, "BottomRight", 6, 2)
	f.text:SetJustifyH("LEFT")
	local title = f:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	title:SetPoint("TopLeft", f.button, "TopRight", 4, -2)
	title:SetText(name)	
	f:SetScript("OnShow", function(self)
		if self ~= f then
			return
		end
		f:initialize()
		f.text:SetText(panel.owner.sets[panel.name][key])
		if not self.stretch then
			self.stretch = 1
			f.button:SetHitRectInsets(0, -(title:GetCenter()+ (title:GetWidth()/2)- f.button:GetCenter()+ (f.button:GetWidth()/2)), 0, 0)
		end
	end)
	f:SetScript("OnHide", function() CloseDropDownMenus() end)
	f.button:SetScript("OnClick", function(self)
		UIDropDownMenu_SetAnchor(f, -4, 6, "TopLeft", f.button, "BottomLeft")
		ToggleDropDownMenu(1, nil, f)
		PlaySound("igMainMenuOptionCheckBoxOn")
	end)
	function f:initialize()
		local owner = panel.owner
		local info = UIDropDownMenu_CreateInfo()
		for i, anchor in ipairs(table) do
			wipe(info)
			info.text = anchor
			info.func = function(item, name)
				owner.sets[panel.name][key] = name
				owner:Layout()
				f.text:SetText(name)
			end
			info.checked = (anchor == owner.sets[panel.name][key])
			info.arg1 = anchor
			UIDropDownMenu_AddButton(info)
		end
	end
	local dropDownList = _G["DropDownList"..1]
	dropDownList.dropdown = f
	dropDownList.shouldRefresh = true
	local prev = panel.lastWidget
	if prev then
		f:SetPoint("Top", panel.lastWidget, "Bottom", 0, -0)
	else
		f:SetPoint("TOPLEFT", 2, -5)
	end
	panel.lastWidget = f
	f.point = {f:GetPoint()}
	f.button:SetHitRectInsets(0, 0 , 0, 0)
	panel.height = panel.height + 24
	f.height = 24
	return f , "checkbutton"
end

local function ShowColorPicker(r, g, b, a, changedCallback)
	ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = nil, nil
	ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = nil, nil, nil
	ColorPickerFrame:SetColorRGB(r,g,b)
	ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = (a ~= nil), a
	ColorPickerFrame.previousValues = {r,g,b,a}
	ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = changedCallback, changedCallback, changedCallback
	ColorPickerFrame:Hide() -- Need to run the OnShow handler.
	ColorPickerFrame:Show()
end

function Menu.ColorPicker(panel, name, key) --this thing needs help.
	local button = CreateFrame("Button", NAME..name, panel)
	NAME = NAME + 1
	button:SetSize(24, 24)
	button:SetHitRectInsets(0,-75,0,0)
	local text = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	text:SetText(name)
	text:SetPoint("Left", button, "Right", 5, 0)
	button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square")
	button:GetHighlightTexture():SetBlendMode("ADD")
	button:SetNormalTexture("Interface\\Tooltips\\UI-Tooltip-Background")
	local border = button:CreateTexture(nil, "OVERLAY")
	border:SetAllPoints(button)
	border:SetTexture("Interface\\BUTTONS\\UI-Quickslot2")
	border:SetTexCoord(.2,.8,.2,.8)
	button:SetScript("OnShow", function(self)
		local saved = panel.owner.sets[string.lower(panel.name)][key]
		if saved then
			button:GetNormalTexture():SetVertexColor(saved.r, saved.g, saved.b, saved.a)
		end
	end)
	button:SetScript("OnClick", function(self)
		local saved = panel.owner.sets[string.lower(panel.name)][key]
		local r, g, b, a = saved.r, saved.g, saved.b, saved.a
		ShowColorPicker(r, g, b, a, function(restore)
			local newR, newG, newB, newA
			if restore then
				newR, newG, newB, newA = unpack(restore)
			else
				newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB()
			end
			saved.r, saved.g, saved.b, saved.a = newR, newG, newB, newA
			self:GetScript("OnShow")(self)
			panel.owner:Layout()
		end)
	end)
	local prev = panel.lastWidget
	if prev then
		button:SetPoint("TopLeft",  prev, "BottomLeft", 0, -2)
	else
		button:SetPoint("TOPLEFT", 2, 0)
	end
	panel.lastWidget = button
	panel.height = panel.height + 24
	button.height = 24
	return button , "checkbutton"
end

function Menu.Media(panel, name, key, handler, mediaType)--panel, "Background", "Background", "GetBackground", "SetBackground"
	local button = bar.master.MediaPanel:NewMediaButton(panel, name, mediaType)
	local handler = string.lower(panel.name)
	button.get = function()
		local owner = panel.owner
		return owner.sets[handler][key]
	end
	button.set = function(...)
		local owner = panel.owner
		owner.sets[handler][key] = ...
		owner:Layout()
	end
	local medi
	button:SetScript("OnClick", function()
			medi = bar.master.MediaPanel:Display(panel, button, mediaType)
			if medi and (medi.holding == button) then
				medi.get = function() return button.get() end
				medi.set = function(...) return button.set(...) end
				medi:UpdateList()
				medi:Show()
			end
		end)
		button.height = 27
	return button , "checkbutton"
end

local function layout(panel, page)

	page.checkbuttonBox = page.checkbuttonBox or CreateFrame("Frame", nil, page)
	page.checkbuttonBox:SetSize(10, 10)
	page.checkbuttonBox:SetPoint("TopLeft", panel, 0, -15)

	page.sliderBox = page.sliderBox or CreateFrame("Frame", nil, page)
	page.sliderBox:SetPoint("TopLeft", page.checkbuttonBox, "BottomLeft")
	page.sliderBox:SetSize(10, 10)

	for type, toggles in pairs(page.toggles) do
		for i, piece in pairs(toggles) do
			piece:ClearAllPoints()
			if type == "checkbutton" then
				if i ~= 1 then
					piece:SetPoint("TopLeft", toggles[i-1], "BottomLeft", 0, -2)
					page.checkbuttonBox:SetPoint("BottomRight", piece)
				else
					piece:SetPoint("TOPLEFT", panel, 2, -25)
					page.checkbuttonBox:SetPoint("BottomRight", piece)
				end
			elseif type == "slider" then
				if i ~= 1 then
					piece:SetPoint("TOP", toggles[i-1], "Bottom", 0, -12)
				else
					piece:SetPoint("TopLeft", page.sliderBox, 0, -15)
				end
			end
		end
	end
end

local function NewPage(panel, title, options)
	local page = CreateFrame("Frame", nil,panel)
	page:SetPoint("TopLeft", panel:GetParent(), 0, -15)
	page:SetPoint("BottomRight", panel:GetParent(), 0, -15)
	page.id = title
	
	
	page.toggles = page.toggles or {checkbutton = {}, slider = {}}

	page.title = title
	tinsert(panel.pages, page)
	page:Hide()
	for name, info in pairs(options) do
		local func = Menu[info.kind]
		local o, type = func(panel, info.name, info.key, info.min or info.table or info.handler, info.max or info.mediaType or info.func, info.step)
		o:SetParent(page)
		page.toggles[type] = page.toggles[type] or {}
		tinsert(page.toggles[type], o)
	end
	layout(panel, page)
	return page
end


local layers = {
	"BACKGROUND",
	"LOW",
	"MEDIUM",
	"HIGH",
	"DIALOG",
	"FULLSCREEN",
	"FULLSCREEN_DIALOG",
	"TOOLTIP",
}



local frameLayer = {
						name = "Frame Layer",
						kind = "Slider",
						key = "frameLayer",
						min = 1,
						max = 100,
					}
local frameStrata = {
						name = "Frame Strata",
						kind = "Slider",
						key = "frameStrata",
						min = 1,
						max = 8,
					}

local function SetLayer(self)
	local lay = layers[self.sets.frameStrata]
	self:SetFrameStrata(lay)
	self:SetFrameLevel(self.sets.frameLayer)
end
					
function AddGlobals(frame, info)
	if not frame.allAdded then
		frame.allAdded  = true
	else
		return
	end


	local def = frame.defaults
	if def then
		def.frameLayer = 1
		def.frameStrata = 2
		frame.sets.frameLayer = frame.sets.frameLayer or def.frameLayer
		frame.sets.frameStrata = frame.sets.frameStrata or def.frameStrata

	end
	local adv = info

		tinsert(adv, {
			name = "Frame Layer",
			kind = "Slider",
			key = "frameLayer",
			min = 1,
			max = 100,
			panel = "advanced",
		})
		tinsert(adv, {
			name = "Frame Strata",
			kind = "Slider",
			key = "frameStrata",
			min = 1,
			max = 8,
			panel = "advanced",
		})

end

bar.AddGlobals = AddGlobals



local PanelSelector
function Menu.NewMenu(self, menu, name, options, test)
	PanelSelector = PanelSelector or Dominos:GetOptions().PanelSelector
	local panel = menu:NewPanel(string.lower(name))
	panel.name = string.lower(name)
	
	local selector = PanelSelector:New(panel)
	selector:SetPoint("TopLeft", panel:GetParent())
	selector:SetPoint("TopRight", panel:GetParent())
	selector:SetHeight(15)
	panel.pages = panel.pages or {}
	
	function selector:OnSelect(id)
		for i, page in pairs(panel.pages) do
			if page.id == id then
				page:Show()
			else
				page:Hide()
			end
		end
	end
	local options = options
		local p = {}
		local trace = {}
		local adv
		for i , info in pairs(options) do
			local title = string.lower(info.panel)
			if title then
				trace[title] = trace[title]
				local t = trace[title]
				if title == "basic" then
					if not trace[title] then
						trace[title] = {}
						tinsert(p , 1, {title= title, info = trace[title]})
					end
					tinsert(trace[title], info)
				else
					if not trace[title] then
						if (title ~= "advanced") and (#p > 1)then
							trace[title] = {}
							tinsert(p, #p, {title= title, info = trace[title]})
						else
							trace[title] = {}
							tinsert(p, {title= title, info = trace[title]})
						end
					end
					if title == "advanced" then
						adv = true
					end
					tinsert(trace[title], info)
				end
			end
		end
		for i, info in pairs(p) do
			selector:AddPanel(info.title)
			if info.title == "advanced" then
				AddGlobals(self, info.info)
			end
			tinsert(panel.pages, NewPage(panel, info.title, info.info))
		end
	if not adv then
		selector:AddPanel("advanced")
		local info = {
				{
					name = "Frame Layer",
					kind = "Slider",
					key = "frameLayer",
					min = 1,
					max = 100,
					panel = "advanced",
				},
				{
					name = "Frame Strata",
					kind = "Slider",
					key = "frameStrata",
					min = 1,
					max = 8,
					panel = "advanced",
				}
		
		}
		
					tinsert(panel.pages, NewPage(panel, "advanced", info))

		
	end
		
	selector:Select("basic")
end
